'use strict';

exports.__esModule = true;

var _class7, _temp3;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _MarkdownRender = require('./MarkdownRender');

var _MarkdownConfig = require('./MarkdownConfig');

var _MarkdownConfig2 = _interopRequireDefault(_MarkdownConfig);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var _uuid = require('uuid/v4');

var Parser = function () {
	Parser.prototype.toString = function toString() {
		return this.name;
	};

	function Parser() {
		_classCallCheck(this, Parser);

		this.name = 'Parser';
	}

	Parser.prototype.render = function render() {
		return null;
	};

	return Parser;
}();

var BlockParser = function (_Parser) {
	_inherits(BlockParser, _Parser);

	function BlockParser() {
		_classCallCheck(this, BlockParser);

		var _this = _possibleConstructorReturn(this, _Parser.call(this));

		_this.inline_tree = [];
		_this.comps = {};
		_this.name = 'BlockParser';
		_this.md = null;
		_this.inlines = [CodeInline, ImageInline, LinkInline, StrongInline, EmInline];
		return _this;
	}

	BlockParser.prototype.bindMarkdown = function bindMarkdown(md) {
		this.md = md;
	};

	BlockParser.prototype.bindComp = function bindComp(uuid, inline) {
		this.comps[uuid] = inline;
	};

	BlockParser.prototype.add = function add(inline) {
		this.inline_tree.push(this.parseInline(inline));
	};

	BlockParser.prototype.parseInline = function parseInline(line) {
		var ret = line;
		for (var _iterator = _MarkdownConfig2.default.InlineParsers, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
			var _ref;

			if (_isArray) {
				if (_i >= _iterator.length) break;
				_ref = _iterator[_i++];
			} else {
				_i = _iterator.next();
				if (_i.done) break;
				_ref = _i.value;
			}

			var inline = _ref;

			if (inline.test(ret)) {
				ret = inline.parse(ret, this);
			}
		}
		return ret;
	};

	BlockParser.prototype.renderInline = function renderInline() {
		var result = [];
		for (var _iterator2 = this.inline_tree, _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
			var _ref2;

			if (_isArray2) {
				if (_i2 >= _iterator2.length) break;
				_ref2 = _iterator2[_i2++];
			} else {
				_i2 = _iterator2.next();
				if (_i2.done) break;
				_ref2 = _i2.value;
			}

			var line = _ref2;

			result.push(InlineParser.renderChildren(line, this.md, this.comps));
		}
		return result;
	};

	return BlockParser;
}(Parser);

var InlineParser = function (_Parser2) {
	_inherits(InlineParser, _Parser2);

	function InlineParser() {
		_classCallCheck(this, InlineParser);

		var _this2 = _possibleConstructorReturn(this, _Parser2.call(this));

		_this2.name = 'InlineParser';
		return _this2;
	}

	InlineParser.getPattern = function getPattern() {
		return (/@/
		);
	};

	InlineParser.getUUID = function getUUID() {
		return '{$@' + _uuid().replace(/\-/g, '') + '}';
	};

	InlineParser.parseOne = function parseOne(line, block) {
		return line;
	};

	InlineParser.parse = function parse(line, block) {
		var tmp = line;
		var index = tmp.search(this.getPattern());
		while (index != -1) {
			tmp = this.parseOne(tmp, block);
			index = tmp.search(this.getPattern());
		}
		return tmp;
	};

	InlineParser.renderChildren = function renderChildren(line, md, comps) {
		var result = [];
		var parts = line.split('{@SPLIT@}');
		for (var _iterator3 = parts, _isArray3 = Array.isArray(_iterator3), _i3 = 0, _iterator3 = _isArray3 ? _iterator3 : _iterator3[Symbol.iterator]();;) {
			var _ref3;

			if (_isArray3) {
				if (_i3 >= _iterator3.length) break;
				_ref3 = _iterator3[_i3++];
			} else {
				_i3 = _iterator3.next();
				if (_i3.done) break;
				_ref3 = _i3.value;
			}

			var part = _ref3;

			if (part.match(/^\{\$@.*\}$/)) {
				result.push(comps[part].render(md, comps));
			} else {
				result.push(part);
			}
		}
		return result;
	};

	return InlineParser;
}(Parser);

var HBlock = function (_BlockParser) {
	_inherits(HBlock, _BlockParser);

	function HBlock() {
		_classCallCheck(this, HBlock);

		var _this3 = _possibleConstructorReturn(this, _BlockParser.call(this));

		_this3.level = 1;
		return _this3;
	}

	HBlock.test = function test(line) {
		return (/^#+ .+/.test(line)
		);
	};

	HBlock.parse = function parse(line, lastBlock) {
		var block = new HBlock();
		block.level = line.match(/^#+/)[0].length;
		var l = line.replace(/^#+ /, '').replace(/\s*#*$/, '');
		block.add(l);
		return block;
	};

	HBlock.prototype.render = function render() {
		return _react2.default.createElement(
			_MarkdownRender.H,
			{ level: this.level },
			this.renderInline()
		);
	};

	return HBlock;
}(BlockParser);

var PBlock = function (_BlockParser2) {
	_inherits(PBlock, _BlockParser2);

	function PBlock() {
		_classCallCheck(this, PBlock);

		return _possibleConstructorReturn(this, _BlockParser2.call(this));
	}

	PBlock.test = function test(line) {
		return true;
	};

	PBlock.parse = function parse(line, lastBlock) {
		if (lastBlock instanceof PBlock) {
			lastBlock.add(line);
			return null;
		}
		var block = new PBlock();
		block.add(line);
		return block;
	};

	PBlock.prototype.render = function render() {
		return _react2.default.createElement(
			_MarkdownRender.P,
			null,
			this.renderInline()
		);
	};

	return PBlock;
}(BlockParser);

var UListBlock = function (_BlockParser3) {
	_inherits(UListBlock, _BlockParser3);

	function UListBlock() {
		_classCallCheck(this, UListBlock);

		return _possibleConstructorReturn(this, _BlockParser3.call(this));
	}

	UListBlock.test = function test(line) {
		return (/^[\*\+\-] /.test(line)
		);
	};

	UListBlock.parse = function parse(line, lastBlock) {
		var l = line.replace(/^[\*\+\-] /, '');
		if (lastBlock instanceof UListBlock) {
			lastBlock.add(l);
			return null;
		}
		var block = new UListBlock();
		block.add(l);
		return block;
	};

	UListBlock.prototype.render = function render() {
		return _react2.default.createElement(_MarkdownRender.UList, { list: this.renderInline() });
	};

	return UListBlock;
}(BlockParser);

var OListBlock = function (_BlockParser4) {
	_inherits(OListBlock, _BlockParser4);

	function OListBlock() {
		_classCallCheck(this, OListBlock);

		return _possibleConstructorReturn(this, _BlockParser4.call(this));
	}

	OListBlock.test = function test(line) {
		return (/^\d+\. /.test(line)
		);
	};

	OListBlock.parse = function parse(line, lastBlock) {
		var l = line.replace(/^\d+\. /, '');
		if (lastBlock instanceof OListBlock) {
			lastBlock.add(l);
			return null;
		}
		var block = new OListBlock();
		block.add(l);
		return block;
	};

	OListBlock.prototype.render = function render() {
		return _react2.default.createElement(_MarkdownRender.OList, { list: this.renderInline() });
	};

	return OListBlock;
}(BlockParser);

var QuoteBlock = function (_BlockParser5) {
	_inherits(QuoteBlock, _BlockParser5);

	function QuoteBlock() {
		_classCallCheck(this, QuoteBlock);

		return _possibleConstructorReturn(this, _BlockParser5.call(this));
	}

	QuoteBlock.test = function test(line) {
		return (/^>/.test(line)
		);
	};

	QuoteBlock.parse = function parse(line, lastBlock) {
		var l = line.replace(/^>/, '');
		if (lastBlock instanceof QuoteBlock) {
			lastBlock.text += '\n' + l;
			return null;
		}
		var block = new QuoteBlock();
		block.text = l;
		return block;
	};

	QuoteBlock.prototype.render = function render() {
		return _react2.default.createElement(
			_MarkdownRender.Quote,
			null,
			new MarkdownParser().parse(this.text)
		);
	};

	return QuoteBlock;
}(BlockParser);

var RefBlock = function (_BlockParser6) {
	_inherits(RefBlock, _BlockParser6);

	function RefBlock() {
		_classCallCheck(this, RefBlock);

		return _possibleConstructorReturn(this, _BlockParser6.call(this));
	}

	RefBlock.test = function test(line) {
		return (/^\[.*\]:\s+.*/.test(line)
		);
	};

	RefBlock.parse = function parse(line, lastBlock, parser) {
		var block = new RefBlock();
		block.id = line.match(/^\[.*\]/)[0].replace(/[\[\]]/g, '');
		if (/[\)\'\"]$/.test(line)) {
			block.title = line.match(/[\(\'\"].*[\)\'\"]$/)[0].replace(/[\(\)\'\"]/g, '');
			line = line.replace(/\s*[\(\'\"].*[\)\'\"]$/, '');
		} else {
			block.title = '';
		}
		block.link = line.match(/:\s+.*$/)[0].replace(/^:\s+/, '');
		parser.addRef(block);
		return null;
	};

	RefBlock.prototype.render = function render() {
		return null;
	};

	return RefBlock;
}(BlockParser);

var LinkInline = function (_InlineParser) {
	_inherits(LinkInline, _InlineParser);

	function LinkInline() {
		var _temp, _this9, _ret;

		_classCallCheck(this, LinkInline);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this9 = _possibleConstructorReturn(this, _InlineParser.call.apply(_InlineParser, [this].concat(args))), _this9), _this9.name = 'LinkInline', _temp), _possibleConstructorReturn(_this9, _ret);
	}

	LinkInline.test = function test(line) {
		return (/\[.*?\](\(.*?\)|\s?\[.*?\])/.test(line)
		);
	};

	LinkInline.getPattern = function getPattern() {
		return (/\[.*?\](\(.*?\)|\s?\[.*?\])/
		);
	};

	LinkInline.parseOne = function parseOne(line, block) {
		var tmp = line;
		var inline = new LinkInline();
		var match = tmp.match(this.getPattern())[0];
		inline.text = match.match(/\[.*?\]/)[0].replace(/[\[\]]/g, '');
		if (/\[.*?\]\s?\[.*?\]$/.test(match)) {
			inline.ref = match.substr(match.match(/\[.*?\]/)[0].length).match(/\[.*\]$/)[0].replace(/[\[\]]/g, '');
		} else {
			inline.link = match.match(/\(.*?\)/)[0].replace(/[\(\)]/g, '');
			if (/ [\'\"].*[\'\"]$/.test(inline.link)) {
				inline.title = inline.link.match(/ [\'\"].*[\'\"]$/)[0].replace(/[ \'\"]/, '');
				inline.link = inline.link.replace(/ [\'\"].*[\'\"]$/, '');
			} else {
				inline.title = '';
			}
		}
		var uuid = this.getUUID();
		block.bindComp(uuid, inline);
		return tmp.replace(this.getPattern(), '{@SPLIT@}' + uuid + '{@SPLIT@}');
	};

	LinkInline.prototype.render = function render(md, comps) {
		if (this.ref != undefined & md.refs[this.ref] != undefined) {
			this.link = md.refs[this.ref].link;
			this.title = md.refs[this.ref].title;
		}
		return _react2.default.createElement(
			_MarkdownRender.Link,
			{ to: this.link, title: this.title },
			InlineParser.renderChildren(this.text, md, comps)
		);
	};

	return LinkInline;
}(InlineParser);

var ImageInline = function (_InlineParser2) {
	_inherits(ImageInline, _InlineParser2);

	function ImageInline() {
		var _temp2, _this10, _ret2;

		_classCallCheck(this, ImageInline);

		for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
			args[_key2] = arguments[_key2];
		}

		return _ret2 = (_temp2 = (_this10 = _possibleConstructorReturn(this, _InlineParser2.call.apply(_InlineParser2, [this].concat(args))), _this10), _this10.name = 'ImageInline', _temp2), _possibleConstructorReturn(_this10, _ret2);
	}

	ImageInline.test = function test(line) {
		return (/\!\[.*?\](\(.*?\)|\s?\[.*?\])/.test(line)
		);
	};

	ImageInline.getPattern = function getPattern() {
		return (/\!\[.*?\](\(.*?\)|\s?\[.*?\])/
		);
	};

	ImageInline.parseOne = function parseOne(line, block) {
		var tmp = line;
		var inline = new ImageInline();
		var match = tmp.match(this.getPattern())[0];
		inline.alt = match.match(/\!\[.*?\]/)[0].replace(/[\!\[\]]/g, '');
		if (/\!\[.*?\]\s?\[.*?\]/.test(match)) {
			inline.ref = match.substr(match.match(/\!\[.*?\]/)[0].length).match(/\[.*\]$/)[0].replace(/[\[\]]/g, '');
		} else {
			inline.src = match.match(/\(.*?\)/)[0].replace(/[\(\)]/g, '');
		}
		var uuid = this.getUUID();
		block.bindComp(uuid, inline);
		return tmp.replace(this.getPattern(), '{@SPLIT@}' + uuid + '{@SPLIT@}');
	};

	ImageInline.prototype.render = function render(md, comps) {
		if (this.ref != undefined) {
			this.src = md.refs[this.ref].link;
			this.alt = md.refs[this.ref].title;
		}
		return _react2.default.createElement(_MarkdownRender.Image, { src: this.src, alt: this.alt });
	};

	return ImageInline;
}(InlineParser);

var EmInline = function (_InlineParser3) {
	_inherits(EmInline, _InlineParser3);

	function EmInline() {
		_classCallCheck(this, EmInline);

		return _possibleConstructorReturn(this, _InlineParser3.apply(this, arguments));
	}

	EmInline.test = function test(line) {
		return (/(\*|\_)[^\*\_]*?(\*|\_)/.test(line)
		);
	};

	EmInline.getPattern = function getPattern() {
		return (/(\*|\_)[^\*\_]*?(\*|\_)/
		);
	};

	EmInline.parseOne = function parseOne(line, block) {
		var tmp = line;
		var inline = new EmInline();
		var match = tmp.match(this.getPattern())[0];
		inline.text = match.replace(/[\*\_]/g, '');
		var uuid = this.getUUID();
		block.bindComp(uuid, inline);
		return tmp.replace(this.getPattern(), '{@SPLIT@}' + uuid + '{@SPLIT@}');
	};

	EmInline.prototype.render = function render(md, comps) {
		return _react2.default.createElement(
			_MarkdownRender.Em,
			null,
			InlineParser.renderChildren(this.text, md, comps)
		);
	};

	return EmInline;
}(InlineParser);

var StrongInline = function (_InlineParser4) {
	_inherits(StrongInline, _InlineParser4);

	function StrongInline() {
		_classCallCheck(this, StrongInline);

		return _possibleConstructorReturn(this, _InlineParser4.apply(this, arguments));
	}

	StrongInline.test = function test(line) {
		return (/(\*\*|\_\_)[^\*\_]*?(\*\*|\_\_)/.test(line)
		);
	};

	StrongInline.getPattern = function getPattern() {
		return (/(\*\*|\_\_)[^\*\_]*?(\*\*|\_\_)/
		);
	};

	StrongInline.parseOne = function parseOne(line, block) {
		var tmp = line;
		var inline = new StrongInline();
		var match = tmp.match(this.getPattern())[0];
		inline.text = match.replace(/[\*\_]/g, '');
		var uuid = this.getUUID();
		block.bindComp(uuid, inline);
		return tmp.replace(this.getPattern(), '{@SPLIT@}' + uuid + '{@SPLIT@}');
	};

	StrongInline.prototype.render = function render(md, comps) {
		return _react2.default.createElement(
			_MarkdownConfig2.default.Renders.strong,
			null,
			InlineParser.renderChildren(this.text, md, comps)
		);
	};

	return StrongInline;
}(InlineParser);

var CodeInline = function (_InlineParser5) {
	_inherits(CodeInline, _InlineParser5);

	function CodeInline() {
		_classCallCheck(this, CodeInline);

		return _possibleConstructorReturn(this, _InlineParser5.apply(this, arguments));
	}

	CodeInline.test = function test(line) {
		return (/`.*?`/.test(line)
		);
	};

	CodeInline.getPattern = function getPattern() {
		return (/`.*?`/
		);
	};

	CodeInline.parseOne = function parseOne(line, block) {
		var tmp = line;
		var inline = new CodeInline();
		var match = tmp.match(this.getPattern())[0];
		inline.text = match.replace(/`/g, '');
		var uuid = this.getUUID();
		block.bindComp(uuid, inline);
		return tmp.replace(this.getPattern(), '{@SPLIT@}' + uuid + '{@SPLIT@}');
	};

	CodeInline.prototype.render = function render(md, comps) {
		return _react2.default.createElement(
			_MarkdownRender.Code,
			null,
			InlineParser.renderChildren(this.text, md, comps)
		);
	};

	return CodeInline;
}(InlineParser);

var MarkdownParser = (_temp3 = _class7 = function () {
	MarkdownParser.init = function init() {
		_MarkdownConfig2.default.BlockParsers = [QuoteBlock, RefBlock, HBlock, UListBlock, OListBlock, PBlock];
		_MarkdownConfig2.default.InlineParsers = [CodeInline, ImageInline, LinkInline, StrongInline, EmInline];
		_MarkdownRender.MarkdownRender.init();
		_MarkdownConfig2.default.haveInited = true;
	};

	function MarkdownParser() {
		_classCallCheck(this, MarkdownParser);

		this.block_tree = [];
		this.refs = {};
	}

	MarkdownParser.prototype.parse = function parse(source) {
		if (!_MarkdownConfig2.default.haveInited) {
			MarkdownParser.init();
		}
		var lines = source.split('\n');
		var lastBlock = null;
		this.block_tree = [];
		for (var _iterator4 = lines, _isArray4 = Array.isArray(_iterator4), _i4 = 0, _iterator4 = _isArray4 ? _iterator4 : _iterator4[Symbol.iterator]();;) {
			var _ref4;

			if (_isArray4) {
				if (_i4 >= _iterator4.length) break;
				_ref4 = _iterator4[_i4++];
			} else {
				_i4 = _iterator4.next();
				if (_i4.done) break;
				_ref4 = _i4.value;
			}

			var line = _ref4;

			for (var _iterator6 = _MarkdownConfig2.default.BlockParsers, _isArray6 = Array.isArray(_iterator6), _i6 = 0, _iterator6 = _isArray6 ? _iterator6 : _iterator6[Symbol.iterator]();;) {
				var _ref6;

				if (_isArray6) {
					if (_i6 >= _iterator6.length) break;
					_ref6 = _iterator6[_i6++];
				} else {
					_i6 = _iterator6.next();
					if (_i6.done) break;
					_ref6 = _i6.value;
				}

				var _block = _ref6;

				if (_block.test(line)) {
					var tmp = _block.parse(line, lastBlock, this);
					if (tmp != null) {
						lastBlock = tmp;
						lastBlock.bindMarkdown(this);
						this.block_tree.push(lastBlock);
					}
					break;
				}
			}
		}
		var result = [];
		for (var _iterator5 = this.block_tree, _isArray5 = Array.isArray(_iterator5), _i5 = 0, _iterator5 = _isArray5 ? _iterator5 : _iterator5[Symbol.iterator]();;) {
			var _ref5;

			if (_isArray5) {
				if (_i5 >= _iterator5.length) break;
				_ref5 = _iterator5[_i5++];
			} else {
				_i5 = _iterator5.next();
				if (_i5.done) break;
				_ref5 = _i5.value;
			}

			var block = _ref5;

			result.push(block.render());
		}
		return result;
	};

	MarkdownParser.prototype.addRef = function addRef(block) {
		this.refs[block.id] = {
			link: block.link,
			title: block.title
		};
	};

	return MarkdownParser;
}(), _class7.blocks = [QuoteBlock, RefBlock, HBlock, UListBlock, OListBlock, PBlock], _temp3);
exports.default = MarkdownParser;
module.exports = exports['default'];